#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_DVMRP.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE DVMRP packets.  Supports DVMRP version 3, decoding
#       of pervious DVMRP versions may be inaccuate.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      23 Jul 1997
#
#  MODIFICATION HISTORY:
#
#
#
INCLUDE ddlinc.ddl

#
# DVMRP Packet Offsets
# DVMRP packets are sent with the IGMP protocol type, the IGMP decode is repeated here.
#
CONSTANT
	DVMRP_type		= 0,
	DVMRP_code		= 1,
	DVMRP_checksum		= 2,
	DVMRP_capabilities	= 5,
	DVMRP_minor		= 6,
	DVMRP_major		= 7,
	DVMRP_genid		= 8,
	DVMRP_probe_sizeof	= 12,
	DVMRP_route_sizeof	= 8,
	DVMRP_ask_sizeof	= 8,
	DVMRP_rsp_sizeof	= 8,
	DVMRP_graft_sizeof	= 16,
	DVMRP_prune_sizeof	= 20;

CONSTANT
	DVMRP_route_mask1	= 0,
	DVMRP_route_mask2	= 2,
	DVMRP_route_mask3	= 3;

CONSTANT
	DVMRP_local_addr	= 0,
	DVMRP_local_metric	= 4,
	DVMRP_local_threshold	= 5,
	DVMRP_local_flags	= 6,
	DVMRP_local_ncount	= 7;

CONSTANT
	DVMRP_graft_source	= 8,
	DVMRP_graft_group	= 12;

CONSTANT
	DVMRP_prune_source	= 8,
	DVMRP_prune_group	= 12,
	DVMRP_prune_lifetime	= 16;

#
# Packet Types
#
CONSTANT
	IGMP_dvmrp		= 0x13,
	DVMRP_probe 		= 1,
	DVMRP_route		= 2,
	DVMRP_ask_neighbors	= 5,
	DVMRP_neighbors_rsp	= 6,
	DVMRP_prune		= 7,
	DVMRP_graft		= 8,
	DVMRP_graft_ack		= 9;

#
# Capabilities
#
FLAG_TABLE dvmrpCapabilities {
	TITLE = "Capabilities",
	OUT_OF_RANGE = "?"
	} =
		0x01 = "Leaf",
		0x02 = "Prune",
		0x04 = "GenID",
		0x08 = "Mtrace",
		0x10 = "Snmp"
		;

#
# Flags
#
FLAG_TABLE dvmrpFlags {
	TITLE = "    Flags",
	OUT_OF_RANGE = "?"
	} =
		0x01 = "Tunnel",
		0x02 = "SourceRoute",
		0x08 = "Down",
		0x10 = "Disabled",
		0x40 = "Leaf"
		;


# ++++++++++++++++++++++++++ Route Update +++++++++++++++++++++++++++++++++++++++++
# Route update packet.  The route formats are tricky, each route section has three
# mask bytes, the number of non-zero masks determine the number of octets of source
# network.  The last network in a section is indicated by the high-order bit in
# the metric being set.
#

#
# DECODE: d_dvmrp_source and d_dvmrp_metric
# Decode the source network and metric.
# Input:
#	CO = source octet
#	$12 = number of octets in source network number
#	$14 = subnet mask
# Output:
#	$13 = 1 if last metric, 0 otherwise 
#
DECLARE PST pst_dvmrp_source;
DECODE d_dvmrp_metric(0) {
	PLEVEL = HL_DVMRP,
	NEXT = pst_dvmrp_source,
	CONSUME = 1
	} =
		decimal( bitfield_byte( 7, 0, 0 ), 0, "    Metric" ),
		$13 = bitfield_byte( 1, 7, 0 )
		;

DECODE d_dvmrp_source(0) { 
	PLEVEL = HL_DVMRP,
	NEXT = d_dvmrp_metric,
	CONSUME = $12
	} =
		ip_address( long( 0 ) & $14, DONT_TRANSLATE, "    Network" )
		;

#
# PST: pst_dvmrp_source
# Output one network with its metric.
# Input:
#	CO = source octet
#	$12 = number of octets in source network number
#	$13 = 1 if end of list, 0 otherwise
#	$14 = subnet mask
#
DECLARE DECODE d_dvmrp_route_section;
PST pst_dvmrp_source( $13 ) {
	DEFAULT = d_dvmrp_route_section
	} =
		d_dvmrp_source
		;

#
# DECODE: d_dvmrp_route_section
# Calculate the size of the source network addresses (in bytes) by counting the
# number of non-zero mask bytes and store the value in $12.
# Input:
# 	CO = route section
# Output:
#	$12 = number of octet bytes
#	$13 = metric MSB
#	$14 = network mask
#
DECODE d_dvmrp_route_section(0) {
	PLEVEL = HL_DVMRP,
	CONSUME = 3,
	NEXT = pst_dvmrp_source
	} =

		# Count the number of source network octets and store in $12, note
		# that the ne() routine access a network byte order longword.  Also
		# derive the network mask and store in $14.
		#
		$12 = 1 +
			bfne( 8, 24, DVMRP_route_mask1 ) +
			bfne( 8, 16, DVMRP_route_mask1 ) +
			bfne( 8, 8,  DVMRP_route_mask1 ),
		$14 = bitfield_long( 24, 8,  DVMRP_route_mask1 ) + 0xFF000000,
		$13 = 0,

		nl(0),
		ip_address( $14, DONT_TRANSLATE, "Routes for subnet mask" )
		;

#
# DECODE d_dvmrp_route
# Decode the fixed portion of the DVMRP route message.
#
DECODE d_dvmrp_route( DVMRP_route ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Route Report",
	BRIEF_TITLE = "DVMRP Report",
	CONSUME = DVMRP_route_sizeof,
	NEXT = d_dvmrp_route_section
	} =

		$1 = remaining(),
		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, ($1 << 16) + DVMRP_checksum, "Checksum (verified)" ),
		decimal( $1, 0, "Packet length" )
		;

#
# ----------------------------------------------------------------------------------


# +++++++++++++++++++++++++++++ Probe Packet ++++++++++++++++++++++++++++++++++++++
#
# PST: pst_dvmrp_neighbors
# Loop through the neighbors list
# Input:
#   CO =  neighbors list
#   $10 = number of neighbors
#
DECLARE PST pst_dvmrp_neighbors;

DECODE d_dvmrp_neighbors( 0 ) {
	PLEVEL = HL_DVMRP,
	NEXT = pst_dvmrp_neighbors,
	CONSUME = 4
	} =
		ip_address( long( 0 ), TRANSLATE, " " )
		;

PST pst_dvmrp_neighbors( $10 ) {
	DEFAULT = d_dvmrp_neighbors
	} =
		;

#
# DECODE: d_dvrmp_probe_2
# Used to decode the generation-id and neighbor count for dvmrp V3 probe packets,
# this should never be reached for pre-V3 packets (which don't have a generation-id
# and neighbor list).
#
DECODE d_dvmrp_probe_2( 0 ) {
	PLEVEL = HL_DVMRP,
	CONSUME = 4,
	NEXT = pst_dvmrp_neighbors
	} =

		hex( long( DVMRP_genid - 8 ), 0, "Generation ID" ),
		decimal( $10, 0, "Number of neighbors" )
		;

# 
# DECODE: d_dvmrp_probe
# This packet has a variable number of neighbors listed in it.   The $10 variable is
# set to contain the number of neighbors.
#
DECODE d_dvmrp_probe( DVMRP_probe ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Probe",
	CONSUME = DVMRP_probe_sizeof - 4,
	NEXT = d_dvmrp_probe_2
	} = 

		$1 = remaining(),
		$10 = ($1 - DVMRP_probe_sizeof) / 4,

		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, ($1 << 16) + DVMRP_checksum, "Checksum (verified)" ),
		flags( byte( DVMRP_capabilities ), 0, dvmrpCapabilities ),
		;


#
# ----------------------------------------------------------------------------------


#
# DECODE: d_dvmrp_ask
# Ask Neighbors 2 packet.
#
DECODE d_dvmrp_ask( DVMRP_ask_neighbors ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Ask Neighbors 2",
	BRIEF_TITLE = "DVMRP Ask",
	CONSUME = DVMRP_ask_sizeof
	} =

		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, (remaining() << 16) + DVMRP_checksum, "Checksum (verified)" )
		;


# ++++++++++++++++++++++++++++  Neighbor Response Packet ++++++++++++++++++++++++++
#
# DECODE: d_dvmrp_rsp_nbr
# Input:
#	CO = neighbor section
#
DECLARE PST pst_dvmrp_rsp_nbrs;
DECODE d_dvmrp_rsp_nbr(0) {
	PLEVEL = HL_DVMRP,
	CONSUME = 4,
	NEXT = pst_dvmrp_rsp_nbrs
	} =
		$15 = $15 - 1,
		ip_address( long( 0 ), TRANSLATE, "    " )
		;

#
# PST: pst_dvmrp_rsp_nbrs
# List all the neighbors in a response packet.
# Input:
#	CO = neighbor section
#       $15 = neighbor count
#
DECLARE DECODE d_dvmrp_local;
PST pst_dvmrp_rsp_nbrs($15) {
	DEFAULT = d_dvmrp_rsp_nbr
	} =
		d_dvmrp_local
		;


#
# DECODE: d_dvmrp_local
# Decode a local address section of a neighbors response message.
# Input:
#	CO = local section
# Output:
#       $15 = neighbor count
#
DECODE d_dvmrp_local(0) {
	PLEVEL = HL_DVMRP,
	CONSUME = 8,
	NEXT = pst_dvmrp_rsp_nbrs
	} =

		$15 = byte( DVMRP_local_ncount ),

		nl(0),
		ip_address( long( DVMRP_local_addr ), TRANSLATE, "Interface address" ),
		decimal( byte( DVMRP_local_metric ), 0, "    Metric" ),
		decimal( byte( DVMRP_local_threshold ), 0, "    Threshold" ),
		flags( byte( DVMRP_local_flags ), 0, dvmrpFlags ),
		decimal( $15, 0, "    Number of neighbors" )
		;


#
# DECODE: d_dvmrp_rsp
# Ask Neighbors Response packet.
#
DECODE d_dvmrp_rsp( DVMRP_neighbors_rsp ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Neighbor Response",
	BRIEF_TITLE = "DVMRP Resp",
	CONSUME = DVMRP_rsp_sizeof,
	NEXT = d_dvmrp_local
	} =

		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, (remaining() << 16) + DVMRP_checksum, "Checksum (verified)" ),
		flags( byte( DVMRP_capabilities ), 0, dvmrpCapabilities )
		;
#
# ----------------------------------------------------------------------------------



# +++++++++++++++++++++++++++++++  Graft Packet  +++++++++++++++++++++++++++++++++++
#
# DECODE: d_dvmrp_graft
# Input:
#	CO = dvmrp header
#
DECODE d_dvmrp_graft( DVMRP_graft ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Graft",
	CONSUME = DVMRP_graft_sizeof
	} =

		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, (remaining() << 16) + DVMRP_checksum, "Checksum (verified)" ),
		ip_address( long( DVMRP_graft_source ), TRANSLATE, "Source address" ),
		ip_address( long( DVMRP_graft_group ), TRANSLATE, "Group address" )
		;

#
# ----------------------------------------------------------------------------------


# ++++++++++++++++++++++++++++  Graft Acknowledgement Packet  ++++++++++++++++++++++
#
# DECODE: d_dvmrp_graft_ack
# Input:
#	CO = dvmrp header
#
DECODE d_dvmrp_graft_ack( DVMRP_graft_ack ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Graft Acknowledgement",
	BRIEF_TITLE = "DVMRP GraftAck",
	CONSUME = DVMRP_graft_sizeof
	} =

		REF d_dvmrp_graft
		;

#
# ----------------------------------------------------------------------------------


# +++++++++++++++++++++++++++++++  Prune Packet  +++++++++++++++++++++++++++++++++++
#
# DECODE: d_dvmrp_prune
# Input:
#	CO = dvmrp header
#
DECODE d_dvmrp_prune( DVMRP_prune ) {
	PLEVEL = HL_DVMRP,
	TITLE = "DVMRP Prune",
	CONSUME = DVMRP_prune_sizeof
	} =

		hex( byte( DVMRP_type ), 0, "Type" ),
		hex( byte( DVMRP_code ), 0, "Code" ),
		print( "Version", "%d.%d", byte( DVMRP_major ), byte( DVMRP_minor ) ),
		ip_checksum( DVMRP_checksum, (remaining() << 16) + DVMRP_checksum, "Checksum (verified)" ),
		ip_address( long( DVMRP_prune_source ), TRANSLATE, "Source address" ),
		ip_address( long( DVMRP_prune_group ), TRANSLATE, "Group address" ),
		decimal( long( DVMRP_prune_lifetime ), 0, "Prune Lifetime (secs)" )
		;

#
# ----------------------------------------------------------------------------------



# 
# PST: pst_dvmrp
# Switch on the DVMRP packet code.
#
PST pst_dvmrp( byte( DVMRP_code ) ) {
	TITLE = "IGMP",
	TYPE = "DVMRP"
	} =
		d_dvmrp_probe,
		d_dvmrp_route,
		d_dvmrp_ask,
		d_dvmrp_rsp,
		d_dvmrp_prune,
		d_dvmrp_graft,
		d_dvmrp_graft_ack
		;

#
# DECODE: d_dvmrp
# Hooks into the IGMP PST, pass control to the pst_igmp PST to switch on the
# DVMRP packet type.
#
DECLARE PST pst_igmp;
DECODE d_dvmrp( IGMP_dvmrp ) {
	PLEVEL = HL_DVMRP,
	ATTACH = pst_igmp,
	NEXT = pst_dvmrp
	} = 
		;
